import { r as registerInstance, c as createEvent, h, f as forceUpdate, g as getElement } from './p-597600b0.js';
import { m as isSymbol, g as getControlSection, b as getGraphDataObj, a as getGraphLayoutObj, j as getElementKey, k as updateConfigData, l as loadAndUpdatePageConfig, u as updateCurrentTabWithMappingsFromOtherTabs, c as updateResultants, d as getOutputSubCategoriesObj, e as updateSelectedOutputSubCategoryId, f as getOutputSection, h as getTabNamesObj, i as updateSelectedTabId, s as showPopup, n as getIDValue } from './p-f53384b7.js';
import { C as COLORS, c as configImageDirPath, D as DIGITAL_FILTER_BLOCK_DIAGRAM_ID } from './p-3ab399ac.js';

/**
 * This method returns the first argument it receives.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Util
 * @param {*} value Any value.
 * @returns {*} Returns `value`.
 * @example
 *
 * var object = { 'a': 1 };
 *
 * console.log(_.identity(object) === object);
 * // => true
 */
function identity(value) {
  return value;
}

/**
 * The base implementation of `_.gt` which doesn't coerce arguments.
 *
 * @private
 * @param {*} value The value to compare.
 * @param {*} other The other value to compare.
 * @returns {boolean} Returns `true` if `value` is greater than `other`,
 *  else `false`.
 */
function baseGt(value, other) {
  return value > other;
}

/**
 * The base implementation of `_.lt` which doesn't coerce arguments.
 *
 * @private
 * @param {*} value The value to compare.
 * @param {*} other The other value to compare.
 * @returns {boolean} Returns `true` if `value` is less than `other`,
 *  else `false`.
 */
function baseLt(value, other) {
  return value < other;
}

/**
 * The base implementation of methods like `_.max` and `_.min` which accepts a
 * `comparator` to determine the extremum value.
 *
 * @private
 * @param {Array} array The array to iterate over.
 * @param {Function} iteratee The iteratee invoked per iteration.
 * @param {Function} comparator The comparator used to compare values.
 * @returns {*} Returns the extremum value.
 */
function baseExtremum(array, iteratee, comparator) {
  var index = -1,
      length = array.length;

  while (++index < length) {
    var value = array[index],
        current = iteratee(value);

    if (current != null && (computed === undefined
          ? (current === current && !isSymbol(current))
          : comparator(current, computed)
        )) {
      var computed = current,
          result = value;
    }
  }
  return result;
}

/**
 * Computes the maximum value of `array`. If `array` is empty or falsey,
 * `undefined` is returned.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Math
 * @param {Array} array The array to iterate over.
 * @returns {*} Returns the maximum value.
 * @example
 *
 * _.max([4, 2, 8, 6]);
 * // => 8
 *
 * _.max([]);
 * // => undefined
 */
function max(array) {
  return (array && array.length)
    ? baseExtremum(array, identity, baseGt)
    : undefined;
}

/**
 * Computes the minimum value of `array`. If `array` is empty or falsey,
 * `undefined` is returned.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Math
 * @param {Array} array The array to iterate over.
 * @returns {*} Returns the minimum value.
 * @example
 *
 * _.min([4, 2, 8, 6]);
 * // => 2
 *
 * _.min([]);
 * // => undefined
 */
function min(array) {
  return (array && array.length)
    ? baseExtremum(array, identity, baseLt)
    : undefined;
}

var indicator = function indicator (i) {
  i = Math.abs(i);
  var cent = i % 100;
  if (cent >= 10 && cent <= 20) return 'th'
  var dec = i % 10;
  if (dec === 1) return 'st'
  if (dec === 2) return 'nd'
  if (dec === 3) return 'rd'
  return 'th'
};

function ordinal (i) {
  if (typeof i !== 'number') throw new TypeError('Expected Number, got ' + (typeof i) + ' ' + i)

  if (!Number.isFinite(i)) return i
  return i + indicator(i)
}

ordinal.indicator = indicator;
var ordinal_1 = ordinal;

/**
 * Returns a function, that, as long as it continues to be invoked, will not
 * be triggered. The function will be called after it stops being called for
 * N milliseconds. If `immediate` is passed, trigger the function on the
 * leading edge, instead of the trailing. The function also has a property 'clear' 
 * that is a function which will clear the timer to prevent previously scheduled executions. 
 *
 * @source underscore.js
 * @see http://unscriptable.com/2009/03/20/debouncing-javascript-methods/
 * @param {Function} function to wrap
 * @param {Number} timeout in ms (`100`)
 * @param {Boolean} whether to execute at the beginning (`false`)
 * @api public
 */
function debounce(func, wait, immediate){
  var timeout, args, context, timestamp, result;
  if (null == wait) wait = 100;

  function later() {
    var last = Date.now() - timestamp;

    if (last < wait && last >= 0) {
      timeout = setTimeout(later, wait - last);
    } else {
      timeout = null;
      if (!immediate) {
        result = func.apply(context, args);
        context = args = null;
      }
    }
  }
  var debounced = function(){
    context = this;
    args = arguments;
    timestamp = Date.now();
    var callNow = immediate && !timeout;
    if (!timeout) timeout = setTimeout(later, wait);
    if (callNow) {
      result = func.apply(context, args);
      context = args = null;
    }

    return result;
  };

  debounced.clear = function() {
    if (timeout) {
      clearTimeout(timeout);
      timeout = null;
    }
  };
  
  debounced.flush = function() {
    if (timeout) {
      result = func.apply(context, args);
      context = args = null;
      
      clearTimeout(timeout);
      timeout = null;
    }
  };

  return debounced;
}
// Adds compatibility for ES modules
debounce.debounce = debounce;

var debounce_1 = debounce;

const adsDigitalFilterCss = ":root{font-family:Roboto}.d-none{display:none}.v-hidden{visibility:hidden}.c-pointer{cursor:pointer}.disabled{cursor:not-allowed;opacity:0.6}.fw-bold{font-weight:bold}::-webkit-scrollbar-track{background-color:var(--color-ti-bg-grey);border-radius:5px}::-webkit-scrollbar{background-color:var(--color-ti-bg-grey);height:5px;width:5px}::-webkit-scrollbar-thumb{background-color:var(--color-ti-text-grey);border-radius:5px}ads-popup::part(popup-container){height:75%;width:75%}.page-header{box-sizing:content-box;display:flex;height:28px;justify-content:space-between;padding-bottom:5px}.page-header .page-title{color:var(--color-ti-text-grey);font-size:24px}.page-content{display:flex;height:calc(100% - 33px)}.page-content .left-section,.page-content .right-section{display:flex;flex-direction:column;gap:2%;height:100%;min-height:670px;padding:2%}.page-content .left-section{background-color:var(--color-ti-primary-white);min-width:540px;width:40%}.page-content .right-section{background-color:#f2f8fa;min-width:720px;width:60%}.popup-btn{align-items:center;cursor:pointer;display:flex;text-decoration:underline}.popup-content{display:flex;flex-direction:column}.popup-content img{margin:auto;max-height:calc(100% - 100px);max-width:calc(100% - 100px)}.popup-content .title{background-color:var(--color-ti-indicator-grey);font-size:1.5rem;font-weight:bold;height:40px;margin:0 auto;padding:5px;text-align:center;width:50%}.section-title{color:var(--color-ti-text-blue);font-size:18px}.section-tabs,.section-outputs{display:flex;flex-direction:column;height:100%;overflow:auto;width:100%}.section-outputs ads-tabs{--selected-tab-color:var(--color-ti-primary-blue-dark)}.section-outputs .tab-content{display:flex;flex-direction:column}.tab-content{height:max-content;overflow:hidden auto;width:100%}.tab-content ads-control{--control-label-width:50%}.tab-content .group-title{color:var(--color-ti-primary-red);font-size:1.2em;margin-top:5%}*{box-sizing:border-box}:host{background-color:var(--color-ti-bg-grey-dark);display:flex;flex-direction:column;height:100%;overflow:auto;padding:2%;width:100%}:host ads-output{margin:3% 5% 0% 5%}:host ads-output:is(.graph){height:calc(100% - 150px)}:host .left-section{min-height:0px}:host .right-section{min-height:0px}:host .input-button-compute-result-container{width:100%;display:flex;align-items:center;justify-content:center;margin-top:10%}:host .min-height{min-height:400px;height:100%}:host .loader{left:0;top:0}:host .error-message-container{display:flex;height:100%;width:100%;justify-content:center;align-items:center;color:red;font-weight:bold;font-size:18px}";

const AdsDigitalFilter = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.setLoaderState = createEvent(this, "setLoaderState", 7);
    this.showToast = createEvent(this, "showToast", 7);
    // ---------- Internal variables ----------
    this.computationAPIs = {};
    this.isComputedValuesInitialized = false;
    this.plotlyDataArrMaxLength = 200000;
    this.plotlyRelayoutDebounceInterval = 1000;
    this.filterResponseGraphPrevRelayoutInfo = {};
    // ----------- Helper methods -------------
    /**
     * To get control element
     * @param controlInfo
     * @param dataPath
     * @returns control element
     */
    this.getControlElement = (controlInfo, dataPath) => {
      return (h("ads-control", { class: controlInfo.id === "computeResult"
          ? "input-button-compute-result-container"
          : "", key: getElementKey(controlInfo), controlInfo: controlInfo, dataPath: dataPath, onSetPathValue: async (ev) => {
          let controlObject = this.config.tabs.configuration.controls;
          let constantObject = this.config.tabs.configuration.constants;
          let defaultValuesUpdated = false;
          let dataPath = ev.detail.dataPath[4];
          let message;
          let restrictedAPIs = [];
          if (constantObject.newInputValues["filterType"] === "") {
            this.updateInputValues(constantObject.oldInputValues, controlObject);
            this.updateInputValues(constantObject.newInputValues, controlObject);
            defaultValuesUpdated = true;
          }
          if (dataPath !== "computeResult" ||
            !controlObject.computeResult.isDisabled) {
            message = "ADC Settings";
            let canUpdateResultant = dataPath === "computeResult" ? true : false;
            constantObject.newInputValues[dataPath] =
              ev.detail.value;
            controlObject.computeResult.isDisabled =
              !this.isInputConfigurationUpdated(constantObject, controlObject) || dataPath === "computeResult";
            controlObject.computeResult.invalid.status =
              !controlObject.computeResult.isDisabled;
            controlObject.computeResult.tooltip = controlObject.computeResult
              .isDisabled
              ? "Results are updated for the current input configuration"
              : "";
            forceUpdate(this.host);
            if (dataPath === "computeResult") {
              if (!controlObject.computeResult.isFilterResponseNeeded) {
                for (let index = 0; index < controlObject.computeResult.restrictedAPIs.length; index++) {
                  restrictedAPIs[index] =
                    this.computationAPIs[controlObject.computeResult.restrictedAPIs[index]];
                  delete this.computationAPIs[controlObject.computeResult.restrictedAPIs[index]];
                }
                message = "Programmable Delay";
              }
              this.setLoaderState.emit({ show: true });
            }
            await updateConfigData.call(this, ev, canUpdateResultant);
            if (dataPath === "computeResult") {
              this.setLoaderState.emit({ show: false });
              setTimeout(() => {
                this.showToast.emit({
                  message: `${message} ${ev.timeStamp}`,
                });
              }, 1000);
              this.updateInputValues(constantObject.oldInputValues, controlObject);
              if (!controlObject.computeResult.isFilterResponseNeeded) {
                for (let index = 0; index < controlObject.computeResult.restrictedAPIs.length; index++) {
                  this.computationAPIs[controlObject.computeResult.restrictedAPIs[index]] = restrictedAPIs[index];
                }
              }
            }
            controlObject.computeResult.isFilterResponseNeeded = controlObject
              .computeResult.isDisabled
              ? false
              : dataPath !== "programmableDelay"
                ? true
                : controlObject.computeResult.isFilterResponseNeeded;
          }
          if (defaultValuesUpdated) {
            for (let index = 0; index < controlObject.computeResult.controlUpdateAPIs.length; index++) {
              delete this.computationAPIs[controlObject.computeResult.controlUpdateAPIs[index]];
            }
          }
        } }));
    };
    /**
     * To get control groups
     * @returns Control groups
     */
    this.getControlSectionGroups = () => {
      const groupsInfo = getControlSection.call(this);
      let groups = [];
      groupsInfo.forEach(group => {
        group.title !== "Compute Result"
          ? groups.push(h("div", null, group.title ? (h("div", { class: "group-title" }, group.title)) : null, ...group.controls))
          : "";
      });
      return groups;
    };
    this.getComputeButon = () => {
      const groupsInfo = getControlSection.call(this);
      let groups = [];
      groupsInfo.forEach(group => {
        group.title === "Compute Result"
          ? groups.push(h("div", null, ...group.controls))
          : "";
      });
      return groups;
    };
    /**
     * To get value for filter response graph
     * @param limits
     * @param metaData
     * @param relayoutInfo
     * @returns Filter response graph value object
     */
    this.getFilterResponseGraphValueObj = (limits, metaData, relayoutInfo) => {
      let xaxisRange;
      let yaxisRange;
      if (relayoutInfo !== undefined && relayoutInfo.resetAxes !== true) {
        if (relayoutInfo.xaxisAutoRange) {
          xaxisRange = [limits.xaxisMin, limits.xaxisMax];
        }
        else {
          xaxisRange = [relayoutInfo.xaxisMin, relayoutInfo.xaxisMax];
        }
        if (relayoutInfo.yaxisAutoRange) {
          yaxisRange = [limits.yaxisMin, limits.yaxisMax];
        }
        else {
          yaxisRange = [relayoutInfo.yaxisMin, relayoutInfo.yaxisMax];
        }
      }
      else {
        let lookupTableMaxLimit = metaData.filterResponse.adjFreqArr.length;
        xaxisRange = [
          0,
          Math.max(100, 5 * Math.ceil(metaData.odr)) <= lookupTableMaxLimit
            ? Math.max(100, 5 * Math.ceil(metaData.odr))
            : lookupTableMaxLimit,
        ];
        yaxisRange = [-140, 0];
      }
      const filterResponseTrace = this.getFilterResponseTrace(metaData, xaxisRange[1], xaxisRange[0]);
      const infoLineYAxisPoints = [
        min(filterResponseTrace.y),
        max(filterResponseTrace.y),
      ];
      return {
        data: getGraphDataObj([
          Object.assign(Object.assign({ color: COLORS.blue, name: "Filter Response" }, filterResponseTrace), { otherOptions: {
              showlegend: false,
            } }),
          {
            color: COLORS.red,
            dash: "dot",
            hoverinfo: "x",
            name: "Normal Mode range",
            otherOptions: {
              legendgroup: "Normal Mode range",
            },
            x: [
              metaData.normalModeRangeFreq.min,
              metaData.normalModeRangeFreq.min,
            ],
            y: infoLineYAxisPoints,
          },
          {
            color: COLORS.red,
            dash: "dot",
            hoverinfo: "x",
            name: "Normal Mode range",
            otherOptions: {
              legendgroup: "Normal Mode range",
              showlegend: false,
            },
            x: [
              metaData.normalModeRangeFreq.max,
              metaData.normalModeRangeFreq.max,
            ],
            y: infoLineYAxisPoints,
          },
          {
            color: COLORS.black,
            dash: "dot",
            hoverinfo: "x",
            name: "Eff. Noise BW",
            x: [
              metaData.effectiveNoiseBandwidth,
              metaData.effectiveNoiseBandwidth,
            ],
            y: infoLineYAxisPoints,
          },
        ].map((lineInfo) => {
          return Object.assign(Object.assign({}, lineInfo), { hoverinfo: lineInfo.hoverinfo || "x+y" });
        })),
        layout: getGraphLayoutObj([], [], {
          gridcolor: COLORS.grey,
          minor: {
            showgrid: true,
          },
          range: xaxisRange,
          title: {
            text: "<b>Frequency (Hz)</b>",
            font: {
              size: 12,
            },
          },
          zeroline: false,
        }, {
          gridcolor: COLORS.grey,
          minor: {
            showgrid: true,
          },
          range: yaxisRange,
          title: {
            text: "<b>Magnitude (dB)</b>",
            font: {
              size: 12,
            },
          },
          zeroline: false,
        }, "", {
          legend: {
            font: {
              size: 11,
            },
            orientation: "h",
            x: 0,
            xanchor: "left",
            y: 1,
            yanchor: "bottom",
          },
          showlegend: true,
        }),
      };
    };
    /**
     * To get filter response trace
     * @param metaData
     * @param maxAdjFreq
     * @param minAdjFreq
     * @returns Filter response trace
     */
    this.getFilterResponseTrace = (metaData, maxAdjFreq, minAdjFreq) => {
      const arrLength = metaData.filterResponse.adjFreqArr.length;
      const adjFreqArrMaxValue = metaData.filterResponse.adjFreqArr.at(-1);
      const adjFreqArrIncrementStep = metaData.filterResponse.adjFreqArr[1] -
        metaData.filterResponse.adjFreqArr[0];
      let adjFreqArr = [];
      let magnitudeArr = [];
      let incrementFactor;
      // ----- Convert adjusted frequency to frequency -----
      const minFreq = metaData.adjFreqToFreq(minAdjFreq);
      const maxFreq = metaData.adjFreqToFreq(maxAdjFreq);
      let minIndex = Math.floor(minFreq * 10) - 1;
      let maxIndex = Math.ceil(maxFreq * 10) - 1;
      // ----- Handle overflow -----
      if (minIndex < 0) {
        minIndex = 0;
      }
      if (maxIndex >= arrLength * 2) {
        maxIndex = arrLength * 2 - 1;
      }
      // ----- Step size & increment value -----
      incrementFactor = Math.ceil((maxIndex - minIndex) / this.plotlyDataArrMaxLength);
      if (incrementFactor === 0) {
        incrementFactor = 1;
      }
      // ----- Update arrays -----
      for (let i = minIndex; i <= maxIndex; i += incrementFactor) {
        if (i < arrLength) {
          adjFreqArr.push(metaData.filterResponse.adjFreqArr[i]);
          magnitudeArr.push(metaData.filterResponse.magnitudeArr[i]);
        }
        else {
          adjFreqArr.push(adjFreqArrMaxValue + (i - arrLength + 1) * adjFreqArrIncrementStep);
          magnitudeArr.push(metaData.filterResponse.magnitudeArr[arrLength - 1 - (i - arrLength)]);
        }
      }
      return {
        x: adjFreqArr,
        y: magnitudeArr,
      };
    };
    /**
     * To get value for CM range graph
     * @param metaData
     * @returns CM range graph value object
     */
    this.getTimeToConversionGraphValueObj = metaData => {
      return {
        data: [
          ...metaData.conversionTimings.map((timing, index) => {
            return {
              hoverinfo: "x",
              line: {
                color: COLORS.black,
              },
              x: [timing],
              y: [0],
            };
          }),
        ],
        layout: getGraphLayoutObj([
          {
            arrowInfo: {
              arrowcolor: COLORS.black,
              arrowhead: 3,
              ax: 0,
              ay: 1,
              axref: "x",
              ayref: "y",
              borderwidth: 1.2,
              showarrow: true,
            },
            fontColor: COLORS.black,
            text: "START",
            textangle: "-90",
            x: 0,
            y: 0,
          },
          ...metaData.conversionTimings.map((timing, index) => {
            return {
              arrowInfo: {
                arrowcolor: COLORS.red,
                arrowhead: 3,
                ax: timing,
                ay: 1,
                axref: "x",
                ayref: "y",
                borderwidth: 1.2,
                showarrow: true,
              },
              fontColor: COLORS.black,
              text: index + 1 + ordinal_1.indicator(index + 1),
              textangle: "-90",
              x: timing,
              y: 0,
            };
          }),
          {
            fontColor: COLORS.red,
            fontSize: 16,
            text: "<b>...</b>",
            xanchor: "left",
            x: metaData.conversionTimings.at(-1) + 0.1,
            y: 0.6,
          },
        ], [], {
          gridcolor: COLORS.grey,
          minor: {
            showgrid: true,
          },
          title: {
            text: `<b>Time (${metaData.conversionTimingsUnit})</b>`,
            font: {
              size: 12,
            },
          },
          zeroline: false,
        }, {
          fixedrange: true,
          range: [-0.02, 2],
          showgrid: false,
          showticklabels: false,
          zeroline: false,
        }),
      };
    };
    /**
     * To get output info
     * @param outputInfo Output info object
     * @returns Output element
     */
    this.getOutputElement = (outputInfo, dataPath) => {
      if (outputInfo.outputType === "graph" && outputInfo.metaData) {
        let value;
        let additionalCustomButtons = [];
        switch (outputInfo.id) {
          case "filterResponseGraph":
            value = this.getFilterResponseGraphValueObj(outputInfo.limits, outputInfo.metaData);
            additionalCustomButtons.push("resetAxes");
            break;
          case "timeToConversionGraph":
            value = this.getTimeToConversionGraphValueObj(outputInfo.metaData);
            break;
        }
        outputInfo = Object.assign(Object.assign({}, outputInfo), { value,
          additionalCustomButtons });
      }
      return (h("ads-output", { "selected-device": this.selectedDevice, "tool-name": "digital-filter", id: outputInfo.id, class: outputInfo.outputType, key: getElementKey(outputInfo), outputInfo: outputInfo, dataPath: dataPath, onPlotlyRelayout: debounce_1.debounce(ev => this.plotlyRelayoutHandler(ev), this.plotlyRelayoutDebounceInterval), onSetPathValue: ev => updateConfigData.call(this, ev), selectedPageIndex: this.selectedPageIndex }));
    };
    this.updateInputValues = (inputValues, controlObject) => {
      for (const prop in controlObject) {
        inputValues[prop] = controlObject[prop].value;
      }
    };
    this.isInputConfigurationUpdated = (constantObject, controlObject) => {
      let inputConfigurationUpdated = false;
      for (const prop in controlObject) {
        if (constantObject.oldInputValues[prop] !=
          constantObject.newInputValues[prop] &&
          prop !== "computeResult") {
          inputConfigurationUpdated = true;
          break;
        }
      }
      return inputConfigurationUpdated;
    };
    // ---------- Event handlers ----------
    /**
     * To handle graph layout change
     * @param relayoutInfo
     */
    this.graphLayoutChangeHandler = relayoutInfo => {
      let filterResponseOutputElement;
      let previousRelayoutInfo = Object.entries(this.filterResponseGraphPrevRelayoutInfo);
      // to handle redudant updates
      if (previousRelayoutInfo.length > 0 &&
        previousRelayoutInfo.every(([key, value]) => {
          return relayoutInfo[key] === value;
        })) {
        return;
      }
      filterResponseOutputElement = this.host.shadowRoot.querySelector("#filterResponseGraph");
      if (filterResponseOutputElement) {
        this.filterResponseGraphPrevRelayoutInfo = Object.assign({}, relayoutInfo);
        filterResponseOutputElement.outputInfo = Object.assign(Object.assign({}, filterResponseOutputElement.outputInfo), { value: this.getFilterResponseGraphValueObj(filterResponseOutputElement.outputInfo.limits, filterResponseOutputElement.outputInfo.metaData, relayoutInfo) });
      }
    };
    /**
     * To handle plotly relayout event
     * @param ev
     */
    this.plotlyRelayoutHandler = ev => {
      if (ev.detail.additionalInfo.resetAxes !== true &&
        [
          "xaxis.autorange",
          "xaxis.range[1]",
          "xaxis.range[0]",
          "yaxis.autorange",
          "yaxis.range[1]",
          "yaxis.range[0]",
        ].every(key => ev.detail[key] === undefined)) {
        return;
      }
      this.graphLayoutChangeHandler({
        resetAxes: ev.detail.additionalInfo.resetAxes,
        xaxisAutoRange: ev.detail["xaxis.autorange"],
        xaxisMax: ev.detail["xaxis.range[1]"] !== undefined
          ? ev.detail["xaxis.range[1]"]
          : ev.detail.additionalInfo.xaxisRange[1],
        xaxisMin: ev.detail["xaxis.range[0]"] !== undefined
          ? ev.detail["xaxis.range[0]"]
          : ev.detail.additionalInfo.xaxisRange[0],
        yaxisAutoRange: ev.detail["yaxis.autorange"],
        yaxisMax: ev.detail["yaxis.range[1]"] !== undefined
          ? ev.detail["yaxis.range[1]"]
          : ev.detail.additionalInfo.yaxisRange[1],
        yaxisMin: ev.detail["yaxis.range[0]"] !== undefined
          ? ev.detail["yaxis.range[0]"]
          : ev.detail.additionalInfo.yaxisRange[0],
      });
    };
    this.selectedDevice = undefined;
    this.selectedPageIndex = undefined;
    this.selectedToolName = undefined;
    this.config = undefined;
    this.parameterConfigUpdated = false;
    this.errorMessage = "";
    this.selectedTabId = undefined;
    this.selectedOutputSubCategoryId = undefined;
    this.isShowPopup = false;
  }
  // ---------- Life cycle methods ----------
  componentDidLoad() {
    if (this.config === undefined && this.selectedDevice !== undefined) {
      this.loadPageConfig();
    }
  }
  // ----------- Watch handlers -------------
  loadPageConfig() {
    this.parameterConfigUpdated = false;
    loadAndUpdatePageConfig.call(this, this.selectedToolName, this.selectedDevice);
  }
  onSelectedTabChanged() {
    if (this.config != undefined) {
      updateCurrentTabWithMappingsFromOtherTabs.call(this);
      updateResultants.call(this, true);
    }
  }
  loadPage() {
    if (this.parameterConfigUpdated && this.errorMessage === "") {
      updateResultants.call(this, true);
      forceUpdate(this.host);
    }
  }
  render() {
    return this.config != undefined &&
      this.isComputedValuesInitialized &&
      this.parameterConfigUpdated &&
      this.errorMessage === "" ? ([
      h("div", { class: "page-header" }, h("div", { class: "page-title" }, "Digital Filter Tool"), h("ads-dropdown-menu", { name: "Related links", items: this.config.collaterals })),
      h("div", { class: "page-content" }, h("div", { class: "left-section", id: "left-section" }, h("div", { class: "section-title" }, this.config.label), h("div", { class: "section-tabs" }, h("ads-tabs", { tabNamesObj: getTabNamesObj(this.config.tabs), selectedTabId: this.selectedTabId, selectedPageIndex: this.selectedPageIndex, onUpdateSelectedTab: ev => updateSelectedTabId.call(this, ev), "hide-if-single-tab": true }), h("div", { class: "tab-content" }, this.getControlSectionGroups()), h("div", null, this.getComputeButon())), h("div", null, h("div", { class: "popup-btn", onClick: ev => showPopup.call(this, ev) }, h("gc-widget-icon", { icon: "filled:action:info_outline", size: "s" }), "Click here to see the digital filter block diagram"))), h("div", { class: "right-section", id: "right-section" }, h("div", { class: "section-title" }, "Resultant Calculations"), h("div", { class: "section-outputs" }, this.selectedTabId != undefined && (h("ads-tabs", { tabNamesObj: getOutputSubCategoriesObj.call(this), selectedTabId: this.selectedOutputSubCategoryId, selectedPageIndex: this.selectedPageIndex, onUpdateSelectedTab: ev => updateSelectedOutputSubCategoryId.call(this, ev), "hide-if-single-tab": true })), h("div", { class: "tab-content min-height" }, getOutputSection.call(this))))),
      h("ads-popup", { isShowPopup: this.isShowPopup, onPopupClosed: () => (this.isShowPopup = false), ref: el => (this.popupDiv = el) }, h("div", { class: "popup-content", ref: el => (this.popupContentDiv = el), slot: "content" }, h("div", { class: "title" }, "Digital Filter Block Diagram"), h("img", { alt: "Block diagram", src: `${configImageDirPath}${getIDValue.call(this, DIGITAL_FILTER_BLOCK_DIAGRAM_ID)}` }))),
    ]) : this.parameterConfigUpdated && this.errorMessage !== "" ? (h("div", { class: "error-message-container" }, this.errorMessage)) : ([h("ads-loader", { class: "loader", isShow: true })]);
  }
  get host() { return getElement(this); }
  static get watchers() { return {
    "selectedDevice": ["loadPageConfig"],
    "selectedTabId": ["onSelectedTabChanged"],
    "parameterConfigUpdated": ["loadPage"]
  }; }
};
AdsDigitalFilter.style = adsDigitalFilterCss;

export { AdsDigitalFilter as ads_digital_filter };

//# sourceMappingURL=p-39ddc97d.entry.js.map