/**
 * Function that needs to be exported for getting the LSB Size
 * arguments should be in the same order as it will be passed from the GUI
 * controlData   : Entire Object
 * parameters    : Parameters stored in the JSON file
 * common        : Common Controls in parameters JSON file
 * codeToVoltage : Code to Voltage Controls in parameters JSON file
 * voltageToCode : Voltage to Code Controls in parameters JSON file
 */
export function getLsbSize(
  controlData,
  parameters,
  common,
  codeToVoltage,
  voltageToCode,
) {
  return (
    getFsrValue(controlData, parameters, common, codeToVoltage, voltageToCode) /
    getNumOfCodesValue(parameters)
  );
}

/**
 * Function that needs to be exported for computing dynamic calculation of control values
 * Control related restrictions are done here
 * arguments should be in the same order as it will be passed from the GUI
 * parameters    : Parameters stored in the JSON file
 * common        : Common Controls in parameters JSON file
 * codeToVoltage : Code to Voltage Controls in parameters JSON file
 * voltageToCode : Voltage to Code Controls in parameters JSON file
 */
export function updateControlInfo(
  controlData,
  parameters,
  common,
  codeToVoltage,
  voltageToCode,
) {
  const refRange = common.refRange.value;
  if (refRange) {
    common.refVoltage.nominalValue = 4.096;
  } else {
    common.refVoltage.nominalValue = 2.5;
  }
}

/**
 * Function that needs to be exported for getting the FSR Value
 * arguments should be in the same order as it will be passed from the GUI
 * controlData   : Entire Object
 * parameters     : Parameters stored in the JSON file
 * common        : Common Controls in parameters JSON file
 * codeToVoltage : Code to Voltage Controls in parameters JSON file
 * voltageToCode : Voltage to Code Controls in parameters JSON file
 */
export function getFsrValue(
  controlData,
  parameters,
  common,
  codeToVoltage,
  voltageToCode,
) {
  const refVoltage = parseFloat(common.refVoltage.value);
  const pgaGain = parseInt(common.pgaGain.value);
  const fsrMultiplier = parseInt(parameters.fsrMultiplier);
  const inputRange = parseInt(common.inputRange.value);
  if (common.extendedRange.value) {
    return (fsrMultiplier * 1.25 * inputRange * refVoltage) / pgaGain;
  }
  return (fsrMultiplier * 1 * inputRange * refVoltage) / pgaGain;
}

//Helper Function
function getNumOfCodesValue(parameters) {
  const bitResolution = parseInt(parameters.resolution);
  return 2 ** bitResolution;
}
